part of disposable_test;

composite_disposable_tests() {

  group('-composite_disposable- should', () {

    test('handle null passed for disposable list', () {
      // Arrange
      var exceptionRaised = null;
      IDisposable compositeDisposable = new CompositeDisposable(null);

      // Act
      try {
      compositeDisposable.dispose();
      } catch(ex) {
        exceptionRaised = ex;
      }

      // Assert
      expect(exceptionRaised, null, reason: exceptionRaised.toString());
    });

    test('throw exception if adding null to composite', () {
      // Arrange
      var caughtException = false;
      CompositeDisposable compositeDisposable = new CompositeDisposable();

      // Act
      try {
        compositeDisposable.add(null);
      } catch(ex) {
        caughtException = true;
      }

      // Assert
      expect(caughtException, true);
    });

    test('throw exception if removing null from composite', () {
      // Arrange
      var caughtException = false;
      CompositeDisposable compositeDisposable = new CompositeDisposable();

      // Act
      try {
        compositeDisposable.remove(null);
      } catch(ex) {
        caughtException = true;
      }

      // Assert
      expect(caughtException, true);
    });

    test('throw exception if checking for contains null from composite', () {
      // Arrange
      var caughtException = false;
      CompositeDisposable compositeDisposable = new CompositeDisposable();

      // Act
      try {
        compositeDisposable.contains(null);
      } catch(ex) {
        caughtException = true;
      }

      // Assert
      expect(caughtException, true);
    });

    test('trigger mulitiple callbacks from CompositeDisposable', () {
      // Arrange
      int disposedCount = 0;
      var callback = () { disposedCount++; };
      IDisposable disposable1 = new Disposable(callback);
      IDisposable disposable2 = new Disposable(callback);
      IDisposable disposable3 = new Disposable(callback);

      IDisposable compositeDisposable = new CompositeDisposable([
        disposable1,
        disposable2,
        disposable3]);

      // Act
      compositeDisposable.dispose();

      // Assert
      expect(disposedCount, 3);
    });

    test('dispose items aadded prior to composite disposal with the container', () {
      // Arrange
      int disposedCount = 0;
      var callback = () { disposedCount++; };
      IDisposable disposable1 = new Disposable(callback);
      IDisposable disposable2 = new Disposable(callback);

      CompositeDisposable compositeDisposable = new CompositeDisposable();
      compositeDisposable.add(disposable1);
      compositeDisposable.add(disposable2);
      expect(disposedCount, 0);

      // Act
      compositeDisposable.dispose();

      // Assert
      expect(disposedCount, 2);
    });

    test('dispose all other items even if a callback throws an exception on composite dispose', () {
      // Arrange
      bool caughtException = false;
      int disposedCount = 0;
      var callback = () { disposedCount++; };
      IDisposable disposable1 = new Disposable(callback);
      IDisposable disposable2 = new Disposable(callback);
      IDisposable disposableThrower = new Disposable(() => throw 'error');

      CompositeDisposable compositeDisposable = new CompositeDisposable();
      compositeDisposable.add(disposable1);
      compositeDisposable.add(disposableThrower);
      compositeDisposable.add(disposable2);
      expect(disposedCount, 0);

      // Act
      try {
        compositeDisposable.dispose();
      } catch(ex) {
        caughtException = true;
      }
      // Assert
      expect(disposedCount, 2);
      expect(caughtException, true);
    });

    test('keep failed disposable in composite if a callback throws an exception on composite clear', () {
      // Arrange
      bool caughtException = false;
      int disposedCount = 0;
      var callback = () { disposedCount++; };
      IDisposable disposable1 = new Disposable(callback);
      IDisposable disposable2 = new Disposable(callback);
      IDisposable disposableThrower = new Disposable(() => throw 'error');

      CompositeDisposable compositeDisposable = new CompositeDisposable();
      compositeDisposable.add(disposable1);
      compositeDisposable.add(disposableThrower);
      compositeDisposable.add(disposable2);
      expect(disposedCount, 0);

      // Act
      try {
        compositeDisposable.clear();
      } catch(ex) {
        caughtException = true;
      }
      // Assert
      expect(disposedCount, 2);
      expect(caughtException, true);
    });

    test('keep reference to disposable in error if a callback throws an exception on composite clear', () {
      // Arrange
      var error = null;
      int disposedCount = 0;
      var callback = () { disposedCount++; };
      IDisposable disposable1 = new Disposable(callback);
      IDisposable disposable2 = new Disposable(callback);
      IDisposable disposableThrower = new Disposable(() => throw 'error');

      CompositeDisposable compositeDisposable = new CompositeDisposable();
      compositeDisposable.add(disposable1);
      compositeDisposable.add(disposableThrower);
      compositeDisposable.add(disposable2);
      expect(disposedCount, 0);

      // Act
      try {
        compositeDisposable.clear();
      } catch(ex) {
        error = ex as List;
      }
      // Assert
      expect(error.length, 1);
    });

    test('immediately dispose items added to a CompositeDisposable which was already disposed', () {
      // Arrange
      int disposedCount = 0;
      var callback = () { disposedCount++; };
      IDisposable disposable = new Disposable(callback);

      CompositeDisposable compositeDisposable = new CompositeDisposable();
      compositeDisposable.dispose();
      expect(disposedCount, 0, reason: 'should not have been disposed yet');

      // Act
      compositeDisposable.add(new Disposable(() => disposedCount++));

      // Assert
      expect(disposedCount, 1, reason: 'should have been disposed without calling dispose on the composite');
    });

    test('dispose and remove item from CopositeDisposable when remove is called', () {
      // Arrange
      int disposedCount = 0;
      var callback = () { disposedCount++; };
      IDisposable disposable = new Disposable(callback);

      CompositeDisposable compositeDisposable = new CompositeDisposable([disposable]);
      expect(disposedCount, 0, reason: 'should not have been disposed yet');

      // Act
      compositeDisposable.remove(disposable);

      // Assert
      expect(disposedCount, 1, reason: 'should not have been disposed yet');
    });

    test('dispose and remove all items from CompositeDisposable without triggering a dispose on the composite itself', () {
      // Arrange
      int disposedCount = 0;
      var callback = () { disposedCount++; };
      IDisposable disposable1 = new Disposable(callback);
      IDisposable disposable2 = new Disposable(callback);

      CompositeDisposable compositeDisposable = new CompositeDisposable([disposable1]);

      expect(disposedCount, 0, reason: 'first should not have been disposed yet');

      // Act
      compositeDisposable.clear();
      expect(disposedCount, 1, reason: 'second should not have been disposed yet');
      compositeDisposable.add(disposable2);

      // Assert
      expect(disposedCount, 1, reason: 'second should not have been disposed');
    });



  });

}